@@ -760,36 +760,6 @@ Literals ith_literal(Literals lits, int i)
     return ith_literal(lits->next, i-1);
 }  /* ith_literal */
 
-/*************
- *
- *   true_term()
- *
- *************/
-
-/* DOCUMENTATION
-*/
-
-/* PUBLIC */
-BOOL true_term(Term t)
-{
-  return is_term(t, true_sym(), 0);
-}  /* true_term */
-
-/*************
- *
- *   false_term()
- *
- *************/
-
-/* DOCUMENTATION
-*/
-
-/* PUBLIC */
-BOOL false_term(Term t)
-{
-  return is_term(t, false_sym(), 0);
-}  /* false_term */
-
 /*************
  *
  *   true_clause()
@@ -960,6 +930,8 @@ BOOL clause_ident(Literals lits1, Literals lits2)
     return lits2 == NULL;
   else if (lits2 == NULL)
     return FALSE;
+  else if (lits1->sign != lits2->sign)
+    return FALSE;
   else if (!term_ident(lits1->atom, lits2->atom))
     return FALSE;
   else
