@@ -1112,11 +1112,12 @@ Pterm toks_to_terms(Tok_pos p)
   Pterm first = NULL;
   Pterm last = NULL;
   Pterm new;
+  int start_pos;
 
   while (!done && !error) {
     parenthesized = FALSE;
     t = NULL;
-    int start_pos = p->tok->buf_pos;
+    start_pos = p->tok->buf_pos;
     
     if (p->tok->type == TOK_ORDINARY ||
 	p->tok->type == TOK_SPECIAL ||
@@ -1741,6 +1742,8 @@ void declare_standard_parse_types(void)
   declare_parse_type("\\",       500, INFIX);
   declare_parse_type("^",        500, INFIX);
   declare_parse_type("v",        500, INFIX);
+  declare_parse_type("++",       500, INFIX);
+  declare_parse_type("--",       500, INFIX);
 
   declare_parse_type(not_sym(),    350, PREFIX);
   declare_parse_type("\'",       300, POSTFIX);
