@@ -43,12 +43,23 @@ BOOL initial_substring(char *x, char *y);
 
 BOOL substring(char *x, char *y);
 
-BOOL str_to_int(char *str, int *ip);
-
 void reverse_chars(char *s, int start, int end);
 
-char *itoa(int n, char *s);
-
 int natural_string(char *str);
 
+int char_occurrences(char *s, char c);
+
+char *escape_char(char *s, char c);
+
+BOOL str_to_int(char *str, int *ip);
+
+char *int_to_str(int n, char *s, int size);
+
+BOOL str_to_double(char *s,
+		   double *dp);
+
+char *double_to_str(double d,
+		    char *s,
+		    int size);
+
 #endif  /* conditional compilation of whole file */
