@@ -34,10 +34,12 @@ struct symbol {
   Lrpo_status lrpo_status;      /* for LRPO, LPO, RPO */
   BOOL        skolem;
   BOOL        unfold;
+  BOOL        auxiliary;        /* not part of theory, e.g., in hints only */
+
   /* IF YOU ADD MORE FIELDS, MAKE SURE TO INITIALIZE THEM ! */
 };
 
-#define SYM_TAB_SIZE  5000
+#define SYM_TAB_SIZE  50000
 
 static Plist By_id[SYM_TAB_SIZE];   /* for access by symnum (ID) */
 static Plist By_sym[SYM_TAB_SIZE];  /* for access by string/arity */
@@ -439,6 +441,7 @@ Symbol get_symbol(void)
   p->type = UNSPECIFIED_SYMBOL;
   p->skolem = FALSE;
   p->unfold = FALSE;
+  p->auxiliary = FALSE;
   return(p);
 }  /* get_symbol */
 
@@ -823,6 +826,25 @@ BOOL is_unfold_symbol(int symnum)
   return p->unfold;
 }  /* is_unfold_symbol */
 
+/*************
+ *
+ *   declare_aux_symbols()
+ *
+ *************/
+
+/* DOCUMENTATION
+*/
+
+/* PUBLIC */
+void declare_aux_symbols(Ilist syms)
+{
+  Ilist p;
+  for (p = syms; p; p = p->next) {
+    Symbol s = lookup_by_id(p->i);
+    s->auxiliary = TRUE;
+  }
+}  /* declare_aux_symbols */
+
 /****************************************************************************/
 /* This section is about parse properties of symbols.                       */
 /****************************************************************************/
@@ -1815,6 +1837,26 @@ void skolem_check(BOOL flag)
   Skolem_check = flag;
 }  /* skolem_check */
 
+/*************
+ *
+ *   skolem_ok()
+ *
+ *************/
+
+static
+BOOL skolem_ok(char *name, int arity)
+{
+  if (!Skolem_check)
+    return TRUE;
+  else {
+    Symbol s = lookup_by_sym(name, arity);
+    if (s == NULL)
+      return TRUE;
+    else
+      return s->auxiliary;
+  }
+}  /* skolem_ok */
+
 /*************
  *
  *   next_skolem_symbol()
@@ -1844,7 +1886,7 @@ int next_skolem_symbol(int arity)
       sprintf(name, "%s%d", Skolem_function_prefix, Next_skolem_function);
       Next_skolem_function++;
     }
-  } while (Skolem_check && lookup_by_sym(name,arity));
+  } while (!skolem_ok(name,arity));
   
   {
     int symnum = str_to_sn(name, arity);
