@@ -437,6 +437,9 @@ BOOL kbo(Term alpha, Term beta, BOOL lex_order_vars)
     else
       return occurs_in(beta, alpha);
   }
+  else if (ARITY(alpha) == 1 && ARITY(beta) == 1 &&
+	   SYMNUM(alpha) == SYMNUM(beta))
+    return kbo(ARG(alpha, 0), ARG(beta, 0), lex_order_vars);
   else if (!variables_multisubset(beta, alpha))
     return FALSE;
   else {
@@ -555,6 +558,9 @@ BOOL flat_kbo(Flatterm alpha, Flatterm beta, BOOL lex_order_vars)
     else
       return flat_occurs_in(beta, alpha);
   }
+  else if (ARITY(alpha) == 1 && ARITY(beta) == 1 &&
+	   SYMNUM(alpha) == SYMNUM(beta))
+    return flat_kbo(alpha->next, beta->next, lex_order_vars);
   else if (!flat_variables_multisubset(beta, alpha))
     return FALSE;
   else {
