@@ -31,6 +31,7 @@ typedef struct readlist * Readlist;
 struct readlist {
   char *name;      /* name, as it appears in the input file */
   int type;        /* FORMULAS, TERMS */
+  BOOL auxiliary;
   Plist *p;        /* *pointer* to the Plist */
   Readlist next;
 };
@@ -39,7 +40,7 @@ static Readlist Input_lists = NULL;
 
 static Plist Lex_function_list = NULL;
 static Plist Lex_predicate_list = NULL;
-static Plist Skolem_list = NULL;
+static Plist Skolem_list = NULL;  /* temp store of user-declared skolems */
 
 static char *Program_name = "";
 
@@ -322,6 +323,31 @@ void process_redeclare(Term t, BOOL echo, FILE *fout)
   }
 }  /* process_redeclare */
 
+/*************
+ *
+ *   execute_unknown_action()
+ *
+ *************/
+
+static
+void execute_unknown_action(FILE *fout, int unknown_action, Term t, char *message)
+{
+  if (unknown_action == KILL_UNKNOWN) {
+    fatal_input_error(fout, message, t);
+  }
+  else if (unknown_action == WARN_UNKNOWN) {
+    bell(stderr);
+    fprintf(fout, "WARNING, %s: ", message);
+    fwrite_term_nl(fout,   t);
+    fprintf(stderr, "WARNING, %s: ", message);
+    fwrite_term_nl(stderr, t);
+  }
+  else if (unknown_action == NOTE_UNKNOWN) {
+    fprintf(fout,   "NOTE, %s: ", message);
+    fwrite_term_nl(fout,   t);
+  }
+}  /* execute_unknown_action */
+
 /*************
  *
  *   flag_handler()
@@ -335,24 +361,21 @@ void process_redeclare(Term t, BOOL echo, FILE *fout)
 void flag_handler(FILE *fout, Term t, BOOL echo, int unknown_action)
 {
   int flag = str_to_flag_id(sn_to_str(SYMNUM(ARG(t,0))));
-  if (flag == -1) {
-    if (unknown_action == KILL_UNKNOWN) {
-      fatal_input_error(fout, "Flag not recognized", t);
-    }
-    else if (unknown_action == WARN_UNKNOWN) {
-      bell(stderr);
-      fprintf(fout,   "WARNING, flag not recognized: ");
-      fwrite_term_nl(fout,   t);
-      fprintf(stderr, "WARNING, flag not recognized: ");
-      fwrite_term_nl(stderr, t);
-    }
-    else if (unknown_action == NOTE_UNKNOWN) {
-      fprintf(fout,   "NOTE: flag not recognized: ");
-      fwrite_term_nl(fout,   t);
+  if (flag == -1)
+    execute_unknown_action(fout, unknown_action, t, "Flag not recognized");
+  else {
+    update_flag(fout, flag, is_term(t, "set", 1), echo);
+    if (is_term(ARG(t,0), "arithmetic", 0)) {
+      if (is_term(t,"set",1)) {
+	printf("\n    %% Declaring Mace4 arithmetic parse types.\n");
+	declare_parse_type("+",   490, INFIX_RIGHT);
+	declare_parse_type("--",  480, INFIX);
+	declare_parse_type("*",   470, INFIX_RIGHT);
+	declare_parse_type("/",   460, INFIX);
+	declare_parse_type("mod", 460, INFIX);
+      }
     }
   }
-  else
-    update_flag(fout, flag, is_term(t, "set", 1), echo);
 }  /* flag_handler */
 
 /*************
@@ -367,51 +390,43 @@ void flag_handler(FILE *fout, Term t, BOOL echo, int unknown_action)
 /* PUBLIC */
 void parm_handler(FILE *fout, Term t, BOOL echo, int unknown_action)
 {
-  int val;
-  BOOL ok = term_to_int(ARG(t,1), &val);
-  if (!ok) {
-    int id = str_to_stringparm_id(sn_to_str(SYMNUM(ARG(t,0))));
-    if (id == -1) {
-      if (unknown_action == KILL_UNKNOWN) {
-	fatal_input_error(fout, "Parameter not recognized", t);
-      }
-      else if (unknown_action == WARN_UNKNOWN) {
-	bell(stderr);
-	fprintf(fout,   "WARNING, parm not recognized: ");
-	fwrite_term_nl(fout,   t);
-	fprintf(stderr, "WARNING, parm not recognized: ");
-	fwrite_term_nl(stderr, t);
-      }
-      else if (unknown_action == NOTE_UNKNOWN) {
-	fprintf(fout,   "NOTE: parm not recognized: ");
-	fwrite_term_nl(fout,   t);
-      }
-    }
+  char *name   = sn_to_str(SYMNUM(ARG(t,0)));
+  Term tval  = ARG(t,1);
+
+  int id = str_to_parm_id(name);
+  if (id != -1) {
+    int val;
+    if (term_to_int(tval, &val))
+      assign_parm(id, val, echo);
     else {
-      char *s = sn_to_str(SYMNUM(ARG(t,1)));
-      assign_stringparm(id, s, echo);
+      execute_unknown_action(fout, unknown_action, t,
+			     "Parm needs integer value");      
     }
   }
   else {
-    int parm = str_to_parm_id(sn_to_str(SYMNUM(ARG(t,0))));
-    if (parm == -1) {
-      if (unknown_action == KILL_UNKNOWN) {
-	fatal_input_error(fout, "Parameter not recognized", t);
-      }
-      else if (unknown_action == WARN_UNKNOWN) {
-	bell(stderr);
-	fprintf(fout,   "WARNING, parm not recognized: ");
-	fwrite_term_nl(fout,   t);
-	fprintf(stderr, "WARNING, parm not recognized: ");
-	fwrite_term_nl(stderr, t);
-      }
-      else if (unknown_action == NOTE_UNKNOWN) {
-	fprintf(fout,   "NOTE: parm not recognized: ");
-	fwrite_term_nl(fout,   t);
-      }
+    id = str_to_floatparm_id(name);
+    if (id != -1) {
+      double val;
+      if (term_to_number(tval, &val))
+	assign_floatparm(id, val, echo);
+      else
+	execute_unknown_action(fout, unknown_action, t,
+			       "Floatparm needs integer or double value");
     }
     else {
-      assign_parm(parm, val, echo);
+      id = str_to_stringparm_id(name);
+      if (id != -1) {
+	if (CONSTANT(tval)) {
+	  char *s = sn_to_str(SYMNUM(tval));
+	  assign_stringparm(id, s, echo);
+	}
+	else
+	  execute_unknown_action(fout, unknown_action, t,
+				 "Stringparm needs constant value");
+      }
+      else
+	execute_unknown_action(fout, unknown_action, t,
+			       "Parameter not recognized");
     }
   }
 }  /* parm_handler */
@@ -485,10 +500,18 @@ Readlist readlist_member_wild(Readlist p, int type)
  *************/
 
 /* DOCUMENTATION
+  name:   name of list
+  type:   type of list (FORMULAS, TERMS)
+  aux:    Is it an auxiliary list?  FALSE: the formulas are part of
+          the logical theory.  TRUE: the formulas are not part of the
+          logical theory, e.g., hints.  When Skolem symbols are generated,
+	  it's ok if the symbols already exist in aux lists.
+  l:      pointer to (pointer to) list where objects should be
+          placed (later, when they are actually read).
 */
 
 /* PUBLIC */
-void accept_list(char *name, int type, Plist *l)
+void accept_list(char *name, int type, BOOL aux, Plist *l)
 {
   Readlist p = readlist_member(Input_lists, name, type);
   if (p)
@@ -497,6 +520,7 @@ void accept_list(char *name, int type, Plist *l)
     p = get_readlist();
     p->name = name;
     p->type = type;
+    p->auxiliary = aux;
     *l = NULL;
     p->p = l;
 
@@ -505,6 +529,33 @@ void accept_list(char *name, int type, Plist *l)
   }
 }  /* accept_list */
 
+/*************
+ *
+ *   input_symbols()
+ *
+ *************/
+
+static
+void input_symbols(int type, BOOL aux, Ilist *fsyms, Ilist *rsyms)
+{
+  I2list fsyms_multiset = NULL;
+  I2list rsyms_multiset = NULL;
+  Readlist p;
+  for (p = Input_lists; p; p = p->next) {
+    if (p->type == type && p->auxiliary == aux) {
+      I2list r = relation_symbols_in_formulas(*(p->p));
+      I2list f = function_symbols_in_formulas(*(p->p));
+      rsyms_multiset = multiset_union(rsyms_multiset, r);
+      fsyms_multiset = multiset_union(fsyms_multiset, f);
+    }
+  }
+  *rsyms = multiset_to_set(rsyms_multiset);
+  *fsyms = multiset_to_set(fsyms_multiset);
+  *fsyms = remove_variable_symbols(*fsyms);
+  zap_i2list(fsyms_multiset);
+  zap_i2list(rsyms_multiset);
+}  /* input_symbols */
+
 /*************
  *
  *   symbol_check_and_declare()
@@ -522,27 +573,23 @@ void accept_list(char *name, int type, Plist *l)
 static
 void symbol_check_and_declare(void)
 {
-  Ilist fsyms, rsyms, bad;
-  I2list fsyms_multiset = NULL;
-  I2list rsyms_multiset = NULL;
-  Readlist p;
+  Ilist fsyms_theory, fsyms_aux, fsyms_all, fsyms_aux_only;
+  Ilist rsyms_theory, rsyms_aux, rsyms_all;
+  Ilist bad;
 
-  for (p = Input_lists; p; p = p->next) {
-    if (p->type == FORMULAS) {
-      I2list r = relation_symbols_in_formulas(*(p->p));
-      I2list f = function_symbols_in_formulas(*(p->p));
-      rsyms_multiset = multiset_union(rsyms_multiset, r);
-      fsyms_multiset = multiset_union(fsyms_multiset, f);
-    }
-  }
+  input_symbols(FORMULAS,  FALSE,  &fsyms_theory, &rsyms_theory);
+  input_symbols(FORMULAS,  TRUE,   &fsyms_aux,    &rsyms_aux);
 
-  rsyms = multiset_to_set(rsyms_multiset);  /* set of relation symbols */
-  fsyms = multiset_to_set(fsyms_multiset);  /* set of function symbols */
-  fsyms = remove_variable_symbols(fsyms);
+  fsyms_all = ilist_union(fsyms_theory, fsyms_aux);
+  rsyms_all = ilist_union(rsyms_theory, rsyms_aux);
+
+  fsyms_aux_only = ilist_subtract(fsyms_aux, fsyms_theory);
+
+  declare_aux_symbols(fsyms_aux_only);
 
   /* Check for variables in rsyms. */
 
-  bad = variable_symbols(rsyms);
+  bad = variable_symbols(rsyms_all);
   if (bad) {
     Ilist g;
     String_buf sb = init_string_buf("The following symbols cannot be used as"
@@ -558,7 +605,7 @@ void symbol_check_and_declare(void)
 
   /* Check if any symbol is used as both a relation and function symbol. */
 
-  bad = ilist_intersect(fsyms, rsyms);
+  bad = ilist_intersect(fsyms_all, rsyms_all);
   if (bad) {
     Ilist g;
     String_buf sb = init_string_buf("The following symbols/arities are used as"
@@ -575,7 +622,7 @@ void symbol_check_and_declare(void)
 
   /* Check if any symbol is used with multiple arities. */
 
-  bad = arity_check(fsyms, rsyms);
+  bad = arity_check(fsyms_all, rsyms_all);
   if (bad) {
     Ilist g;
     String_buf sb = init_string_buf("The following symbols are used with"
@@ -593,24 +640,40 @@ void symbol_check_and_declare(void)
   /* Tell the symbol package the functions and relations.
      (needed for ordering the symbols) */
 
-  declare_functions_and_relations(fsyms, rsyms);
+  declare_functions_and_relations(fsyms_all, rsyms_all);
 
-  process_skolem_list(Skolem_list, fsyms);
+  process_skolem_list(Skolem_list, fsyms_theory);
   zap_plist(Skolem_list);
   Skolem_list = NULL;
 
-  process_lex_list(Lex_function_list, fsyms, FUNCTION_SYMBOL);
-  process_lex_list(Lex_predicate_list, rsyms, PREDICATE_SYMBOL);
+  process_lex_list(Lex_function_list, fsyms_all, FUNCTION_SYMBOL);
+  process_lex_list(Lex_predicate_list, rsyms_all, PREDICATE_SYMBOL);
 
   zap_plist(Lex_function_list);
   zap_plist(Lex_predicate_list);
   Lex_function_list = NULL;
   Lex_predicate_list = NULL;
 
-  zap_i2list(fsyms_multiset);
-  zap_i2list(rsyms_multiset);
-  zap_ilist(fsyms);
-  zap_ilist(rsyms);
+  /*
+  p_syms();
+  printf("fsyms_theory: "); p_ilist(fsyms_theory);
+  printf("fsyms_aux: "); p_ilist(fsyms_aux);
+  printf("fsyms_all: "); p_ilist(fsyms_all);
+  printf("fsyms_aux_only: "); p_ilist(fsyms_aux_only);
+  printf("rsyms_theory: "); p_ilist(rsyms_theory);
+  printf("rsyms_aux: "); p_ilist(rsyms_aux);
+  printf("rsyms_all: "); p_ilist(rsyms_all);
+  */
+
+  zap_ilist(fsyms_theory);
+  zap_ilist(fsyms_aux);
+  zap_ilist(fsyms_all);
+  zap_ilist(fsyms_aux_only);
+
+  zap_ilist(rsyms_theory);
+  zap_ilist(rsyms_aux);
+  zap_ilist(rsyms_all);
+
 }  /* symbol_check_and_declare */
 
 /*************
@@ -980,6 +1043,7 @@ Term read_commands(FILE *fin, FILE *fout, BOOL echo, int unknown_action)
       flag_handler(fout, t, echo, unknown_action);
 
       /* SPECIAL CASES: these need action right now! */
+
       if (is_term(ARG(t,0), "prolog_style_variables", 0)) {
 	if (is_term(t,"set",1))
 	  set_variable_style(PROLOG_STYLE);
