@@ -20,19 +20,7 @@
 
 /* Private definitions and types */
 
-/*************
- *
- *   declare_tptp_parse_types()
- *
- *************/
-
-/* DOCUMENTATION
-*/
-
-/* PUBLIC */
-void declare_tptp_parse_types(void)
-{
-  /*
+/*
 
   From Geoff Sutcliffe, Feb 13, 2008:
 
@@ -80,6 +68,8 @@ void declare_tptp_parse_types(void)
   %----Must be weak to allow any formulae on RHS
   :-op(550,xfy,:=).
 
+  --------------------------------------------------------
+
   Correction from Geoff, May 14, 2008:
 
     op(502,xfx,'~|'),
@@ -90,7 +80,77 @@ void declare_tptp_parse_types(void)
     op(505,xfx,<~>),
     op(550,xfx,:=).
 
-  */
+  --------------------------------------------------------
+
+  From Geoff, October 6, 2008:
+
+  I would be quite happy if both [input and output] were equally strict,
+  conforming to the
+  TPTP BNF. However, your other users might be happier if the input were
+  a little more forgiving, and that's why my Prolog operator defs are
+  like they are - more forgiving than the BNF.
+ 
+  I guess you have the following at the moment, which are the "forgiving"
+  operator defs ...
+
+    op(400,fy,!),
+    op(400,fx,?),
+    op(405,xfx,'='),
+    op(450,xfy,:),
+    op(450,fy,~),
+    op(502,xfy,'|'),
+    op(502,xfx,'~|'),
+    op(503,xfy,&),
+    op(503,xfx,~&),
+    op(504,xfx,=>),
+    op(504,xfx,<=),
+    op(505,xfx,<=>),
+    op(505,xfx,<~>),
+
+  Indeed, I solved the current issue by reading your output with my Prolog
+  tool, whose parsing uses those defs, but whose output routines are hand 
+  rolled to output in strict TPTP format. For output, or generally strict, 
+  you can set all the binary connectives to the same precedence, e.g., ...
+
+    op(502,xfy,'|'),
+    op(502,xfx,'~|'),
+    op(502,xfy,&),
+    op(502,xfx,~&),
+    op(502,xfx,=>),
+    op(502,xfx,<=),
+    op(502,xfx,<=>),
+    op(502,xfx,<~>),
+
+  Note the | and & are xfy, which allows, e.g., (a | b | c | d).
+
+  --------------------------------------------------------
+
+  WWM: Ok, I'll separate the types for parsing TPTP and printing TPTP.
+  Parsing TPTP will be flexible, pretty much as before.  Printing
+  will be conservative, using Geoff's second set of op defs above,
+  with the exception that & and | will be INFIX instead of INFIX_RIGHT,
+  (because LADR does not allow different parse types with same precedence).
+  So we'll print, e.g.,  (a & (b & c)) => (d | (e | f)), whereas
+  the TPTP string a & b & c => d | e | f wil be parsed correctly as the
+  same term.
+
+  Also, for ladr_to_tptp, I'll add an command-line option -p that will
+  cause ALL expressions to be parenthesized.
+
+*/
+
+/*************
+ *
+ *   declare_tptp_input_types()
+ *
+ *************/
+
+/* DOCUMENTATION
+*/
+
+/* PUBLIC */
+void declare_tptp_input_types(void)
+{
 
   /* We do not support all of the TPTP operations listed above. */
 
@@ -110,10 +170,7 @@ void declare_tptp_parse_types(void)
   declare_parse_type(":",   450, INFIX_RIGHT);
 
   declare_parse_type("|",   502, INFIX_RIGHT);
-  /* declare_parse_type("~|",  502, INFIX); */
-
   declare_parse_type("&",   503, INFIX_RIGHT);
-  /* declare_parse_type("~&",  503, INFIX); */
 
   declare_parse_type("=>",  504, INFIX);
   declare_parse_type("<=",  504, INFIX);
@@ -124,7 +181,51 @@ void declare_tptp_parse_types(void)
 
   set_cons_char('\0');  /* Don't recognize list cons */
 
-}  /* declare_tptp_parse_types */
+}  /* declare_tptp_input_types */
+
+/*************
+ *
+ *   declare_tptp_output_types()
+ *
+ *************/
+
+/* DOCUMENTATION
+*/
+
+/* PUBLIC */
+void declare_tptp_output_types(void)
+{
+
+  /* We do not support all of the TPTP operations listed above. */
+
+  declare_parse_type(",",   999, INFIX_RIGHT);     /* LADR requirement */
+
+  declare_parse_type("++",  100, PREFIX_PAREN);
+  declare_parse_type("--",  100, PREFIX_PAREN);
+
+  declare_parse_type("!",   400, PREFIX_PAREN);
+  declare_parse_type("?",   400, PREFIX_PAREN);
+
+  declare_parse_type("!=",  405, INFIX);           /* added by McCune */
+  declare_parse_type("=",   405, INFIX);
+
+  declare_parse_type("~",   410, PREFIX);          /* changed by McCune */
+
+  declare_parse_type(":",   450, INFIX_RIGHT);
+
+  declare_parse_type("|",   502, INFIX);  /* NOTE: not INFIX_RIGHT (xfy) */
+  declare_parse_type("&",   502, INFIX);  /* NOTE: not INFIX_RIGHT (xfy) */
+
+  declare_parse_type("=>",  502, INFIX);
+  declare_parse_type("<=",  502, INFIX);
+  declare_parse_type("<=>", 502, INFIX);
+  declare_parse_type("<~>", 502, INFIX);
+
+  /* Other things */
+
+  set_cons_char('\0');  /* Don't recognize list cons */
+
+}  /* declare_tptp_output_types */
 
 /*****************************************************************************
 
@@ -273,11 +374,12 @@ Term tptp3_to_ladr_term(Term t)
     Term q = ARG(t,0);
     Term form = tptp3_to_ladr_term(ARG(t,1));
     Term vars = listterm_reverse(ARG(q,0));
+    char *quant;
 
     if (!proper_listterm(vars))
       fatal_error("tptp_to_ladr: bad quantified variable(s)");
 
-    char *quant = (is_term(q,"!",1) ? all_sym() : exists_sym());
+    quant = (is_term(q,"!",1) ? all_sym() : exists_sym());
 
     while (cons_term(vars)) {
       Term v = ARG(vars, 0);
@@ -581,6 +683,18 @@ BOOL good_tptp_sym(char *s)
   }
 }  /* good_tptp_sym */
 
+/*************
+ *
+ *   good_ladr_sym()
+ *
+ *************/
+
+static
+BOOL good_ladr_sym(char *s)
+{
+  return s[0] != '\'';
+}  /* good_ladr_sym */
+
 /*************
  *
  *   map_for_bad_tptp_syms()
@@ -591,16 +705,30 @@ BOOL good_tptp_sym(char *s)
 */
 
 /* PUBLIC */
-I2list map_for_bad_tptp_syms(Ilist syms)
+I2list map_for_bad_tptp_syms(Ilist syms, BOOL quote_bad_syms)
 {
   if (syms == NULL)
     return NULL;
   else {
-    I2list map = map_for_bad_tptp_syms(syms->next);
+    I2list map = map_for_bad_tptp_syms(syms->next, quote_bad_syms);
     int old_sn = syms->i;
     char *s = sn_to_str(old_sn);
     if (!good_tptp_sym(s)) {
-      int new_sn = fresh_symbol("tptp", sn_to_arity(old_sn));
+      int new_sn;
+      if (quote_bad_syms) {
+	char *escaped = escape_char(s, '\'');
+	int n = strlen(escaped);
+	char *new_str = malloc(n+3);
+	new_str[0] = '\'';
+	strcpy(new_str+1, escaped);
+	new_str[n+1] = '\'';
+	new_str[n+2] = '\0';
+	new_sn = str_to_sn(new_str, sn_to_arity(old_sn));
+	free(new_str);
+	free(escaped);
+      }
+      else
+	new_sn = fresh_symbol("tptp", sn_to_arity(old_sn));
       map = i2list_append(map, old_sn, new_sn);
     }
     return map;
@@ -609,12 +737,56 @@ I2list map_for_bad_tptp_syms(Ilist syms)
 
 /*************
  *
- *   replace_bad_tptp_syms_term()
+ *   map_for_bad_ladr_syms()
  *
  *************/
 
-static
-Term replace_bad_tptp_syms_term(Term t, I2list map)
+/* DOCUMENTATION
+*/
+
+/* PUBLIC */
+I2list map_for_bad_ladr_syms(Ilist syms, BOOL quote_bad_syms)
+{
+  if (syms == NULL)
+    return NULL;
+  else {
+    I2list map = map_for_bad_ladr_syms(syms->next, quote_bad_syms);
+    int old_sn = syms->i;
+    char *s = sn_to_str(old_sn);
+    if (!good_ladr_sym(s)) {
+      int new_sn;
+      if (quote_bad_syms) {
+	char *escaped = escape_char(s, '"');
+	int n = strlen(escaped);
+	char *new_str = malloc(n+3);
+	new_str[0] = '"';
+	strcpy(new_str+1, escaped);
+	new_str[n+1] = '"';
+	new_str[n+2] = '\0';
+	new_sn = str_to_sn(new_str, sn_to_arity(old_sn));
+	free(new_str);
+	free(escaped);
+      }
+      else
+	new_sn = fresh_symbol("ladr", sn_to_arity(old_sn));
+      map = i2list_append(map, old_sn, new_sn);
+    }
+    return map;
+  }
+}  /* map_for_bad_ladr_syms */
+
+/*************
+ *
+ *   replace_bad_syms_term()
+ *
+ *************/
+
+/* DOCUMENTATION
+The given term is used up.
+*/
+
+/* PUBLIC */
+Term replace_bad_syms_term(Term t, I2list map)
 {
   if (VARIABLE(t))
     return t;
@@ -630,10 +802,10 @@ Term replace_bad_tptp_syms_term(Term t, I2list map)
       t = new;
     }
     for (i = 0; i < ARITY(t); i++)
-      ARG(t,i) = replace_bad_tptp_syms_term(ARG(t,i), map);
+      ARG(t,i) = replace_bad_syms_term(ARG(t,i), map);
     return t;
   }
-}  /* replace_bad_tptp_syms_term */
+}  /* replace_bad_syms_term */
 
 /*************
  *
@@ -674,8 +846,7 @@ Term replace_bad_tptp_syms_form(Term t, BOOL clausal, I2list map)
     return t;
   }
   else
-    return replace_bad_tptp_syms_term(t, map);
+    return replace_bad_syms_term(t, map);
 }  /* replace_bad_tptp_syms_form */
 
 
-
