@@ -565,6 +565,39 @@ Term apply_substitute2(Term t, Term beta, Context c_from,
   }
 }  /* apply_substitute2 */
 
+/*************
+ *
+ *    apply_demod()
+ *
+ *    Special-purpose apply for ordinary demodulation.
+ *    Assume every variable in t is instantated by the
+ *    substitution.  Terms that come from instantiating
+ *    variables get the flag set, indicating that the
+ *    term is fully demodulated (assuming inside-out
+ *    demodulation).
+ *    
+ *************/
+
+/* DOCUMENTATION
+*/
+
+/* PUBLIC */
+Term apply_demod(Term t, Context c, int flag)
+{
+  Term b;
+  if (VARIABLE(t)) {
+    b = copy_term(c->terms[VARNUM(t)]);
+    term_flag_set(b, flag);
+  }
+  else {
+    int i;
+    b = get_rigid_term_like(t);
+    for (i = 0; i < ARITY(t); i++)
+      ARG(b,i) = apply_demod(ARG(t,i), c, flag);
+  }
+  return b;
+}  /* apply_demod */
+
 /*************
  *
  *    undo_subst(tr) -- Clear a substitution.
