@@ -32,19 +32,18 @@
 /* Public function prototypes from weight.c */
 
 void init_weight(Plist rules,
-		 int variable_weight,
-		 int constant_weight,
-		 int not_weight,
-		 int or_weight,
-		 int sk_constant_weight,
-		 int prop_atom_weight,
-		 int skolem_penalty,
-		 int nest_penalty,
-		 int depth_penalty,
-		 int var_penalty);
+		 double variable_weight,
+		 double constant_weight,
+		 double not_weight,
+		 double or_weight,
+		 double sk_constant_weight,
+		 double prop_atom_weight,
+		 double nest_penalty,
+		 double depth_penalty,
+		 double var_penalty);
 
-int weight(Term t, Context subst);
+double weight(Term t, Context subst);
 
-int clause_weight(Literals lits);
+double clause_weight(Literals lits);
 
 #endif  /* conditional compilation of whole file */
