@@ -17,7 +17,6 @@
 */
 
 #include "msearch.h"
-
 #include "../ladr/banner.h"
 
 // #define DEBUG
@@ -84,7 +83,8 @@ enum {
   SEARCH_MAX_MODELS,          /* stop */
   SEARCH_MAX_MEGS,            /* stop */
   SEARCH_MAX_TOTAL_SECONDS,   /* stop */
-  SEARCH_MAX_DOMAIN_SECONDS   /* stop */
+  SEARCH_MAX_DOMAIN_SECONDS,  /* stop */
+  SEARCH_DOMAIN_OUT_OF_RANGE  /* stop */
 };
 
 /* Ground terms.  MACE4 operates on ground clauses, which are
@@ -153,6 +153,8 @@ void init_mace_options(Mace_options opt)
   opt->neg_elim_near          = init_flag("neg_elim_near",          TRUE);
   opt->verbose                = init_flag("verbose",                FALSE);
   opt->integer_ring           = init_flag("integer_ring",           FALSE);
+  opt->order_domain           = init_flag("order_domain",           FALSE);
+  opt->arithmetic             = init_flag("arithmetic",             FALSE);
   opt->iterate_primes         = init_flag("iterate_primes",         FALSE);
   opt->iterate_nonprimes      = init_flag("iterate_nonprimes",      FALSE);
   opt->skolems_last           = init_flag("skolems_last",           FALSE);
@@ -181,6 +183,12 @@ void init_mace_options(Mace_options opt)
   flag_stringparm_dependency(opt->iterate_primes,TRUE,opt->iterate,"primes");
   flag_stringparm_dependency(opt->iterate_nonprimes,TRUE,opt->iterate,"nonprimes");
 
+  flag_flag_dependency(opt->integer_ring, TRUE, opt->lnh, FALSE);
+  flag_flag_dependency(opt->order_domain, TRUE, opt->lnh, FALSE);
+  flag_flag_dependency(opt->arithmetic, TRUE, opt->lnh, FALSE);
+
+  flag_parm_dependency(opt->arithmetic,TRUE,opt->selection_order,0);
+
 }  /* init_mace_options */
 
 /*************
@@ -279,27 +287,21 @@ void initialize_for_search(Plist clauses)
 
   init_built_in_symbols();  /* =/2 (and maybe others) */
 
+  /* Maybe initialize for arithmetic. */
+
+  if (flag(Opt->arithmetic))
+    init_arithmetic();
+
   /* Collect data for each symbol. */
 
   Max_domain_element_in_input = -1;
-#if 0
-  for (p = clauses; p != NULL; p = p->next) {
-    Topform c = p->v;
-    Literal lit;
-    for (lit = c->literals; lit != NULL; lit = lit->next) {
-      int x = collect_mace_syms(lit->atom, RELATION);
-      Max_domain_element_in_input = MAX(Max_domain_element_in_input, x);
-    }
-  }
-#else
-  i = collect_mace4_syms(clauses);
+  i = collect_mace4_syms(clauses, flag(Opt->arithmetic));
   Max_domain_element_in_input = MAX(Max_domain_element_in_input, i);
-#endif
 
   if (Max_domain_element_in_input == -1)
-    printf("\n%% There are no domain elements in the input.\n");
+    printf("\n%% There are no natural numbers in the input.\n");
   else
-    printf("\n%% The maximum domain element in the input is %d.\n",
+    printf("\n%% The largest natural number in the input is %d.\n",
 	   Max_domain_element_in_input);
 
   /* Set up map from ordinary symnums to mace symnums. */
@@ -430,11 +432,28 @@ void special_assignments(void)
 	for (i = 0; i < Domain_size; i++)
 	  Cells[X1(s->base,i)].value = Domain[(Domain_size - i) % Domain_size];
       }
+      else if (is_symbol(s->sn, "--", 2)) {
+	for (i = 0; i < Domain_size; i++)
+	  for (j = 0; j < Domain_size; j++)
+	    Cells[X2(s->base,i,j)].value = Domain[((i + Domain_size) - j) %
+						  Domain_size];
+      }
     }
-    if (flag(Opt->lnh)) {
-      printf("\nClearing flag lnh, because the least number heuristic\n"
-	     "is not compatible with flag integer_ring.\n");
-      clear_flag(Opt->lnh, TRUE);
+  }
+  if (flag(Opt->order_domain)) {
+    Symbol_data s;
+    for (s = Symbols; s != NULL; s = s->next) {
+      int i, j;
+      if (is_symbol(s->sn, "<", 2)) {
+	for (i = 0; i < Domain_size; i++)
+	  for (j = 0; j < Domain_size; j++)
+	    Cells[X2(s->base,i,j)].value = (Domain[i<j ? 1 : 0]);
+      }
+      if (is_symbol(s->sn, "<=", 2)) {
+	for (i = 0; i < Domain_size; i++)
+	  for (j = 0; j < Domain_size; j++)
+	    Cells[X2(s->base,i,j)].value = (Domain[i<=j ? 1 : 0]);
+      }
     }
   }
 }  /* special_assignments */
@@ -446,7 +465,7 @@ void special_assignments(void)
  *************/
 
 static
-void check_that_ground_clauses_are_true(void)
+BOOL check_that_ground_clauses_are_true(void)
 {
   Plist g;
   BOOL ok = TRUE;
@@ -459,46 +478,52 @@ void check_that_ground_clauses_are_true(void)
       ok = FALSE;
     }
   }
-  if (!ok)
-    fatal_error("check_that_ground_clauses_are_true, bad model found");
+  return ok;
 }  /* check_that_ground_clauses_are_true */
 
 /*************
  *
- *   we_have_a_model()
+ *   possible_model()
  *
  *************/
 
 static
-int we_have_a_model(void)
+int possible_model(void)
 {
-  static int next_message = 1;
-  Total_models++;
-  Mstats.current_models++;
-
-  check_that_ground_clauses_are_true();  /* in case of a soundness bug */
-
-  if (flag(Opt->return_models)) {
-    Term modelterm = interp_term();
-    Interp model = compile_interp(modelterm, FALSE);
-    zap_term(modelterm);
-    Models = plist_append(Models, model);
+  if (flag(Opt->arithmetic)) {
+    if (!check_with_arithmetic(Ground_clauses))
+      return SEARCH_GO_NO_MODELS;
   }
+  else if (!check_that_ground_clauses_are_true())
+    fatal_error("possible_model, bad model found");
 
-  if (flag(Opt->print_models))
-    print_model_standard(stdout, TRUE);
-  else if (flag(Opt->print_models_tabular))
-    p_model(FALSE);
-  else if (next_message == Total_models) {
-    printf("\nModel %d has been found.\n", Total_models);
-    next_message *= 10;
+  {
+    static int next_message = 1;
+    Total_models++;
+    Mstats.current_models++;
+
+    if (flag(Opt->return_models)) {
+      Term modelterm = interp_term();
+      Interp model = compile_interp(modelterm, FALSE);
+      zap_term(modelterm);
+      Models = plist_append(Models, model);
+    }
+
+    if (flag(Opt->print_models))
+      print_model_standard(stdout, TRUE);
+    else if (flag(Opt->print_models_tabular))
+      p_model(FALSE);
+    else if (next_message == Total_models) {
+      printf("\nModel %d has been found.\n", Total_models);
+      next_message *= 10;
+    }
+    fflush(stdout);
+    if (parm(Opt->max_models) != -1 && Total_models >= parm(Opt->max_models))
+      return SEARCH_MAX_MODELS;
+    else
+      return SEARCH_GO_MODELS;
   }
-  fflush(stdout);
-  if (parm(Opt->max_models) != -1 && Total_models >= parm(Opt->max_models))
-    return SEARCH_MAX_MODELS;
-  else
-    return SEARCH_GO_MODELS;
-}  /* we_have_a_model */
+}  /* possible_model */
 
 /*************
  *
@@ -635,7 +660,7 @@ int search(int max_constrained, int depth)
   else {
     id = select_cell(max_constrained);
     if (id == -1) {
-      rc = we_have_a_model();
+      rc = possible_model();
       return rc;
     }
     else {
@@ -701,8 +726,14 @@ int mace4n(Plist clauses, int order)
   Variable_style save_style = variable_style();
   set_variable_style(INTEGER_STYLE);
 
-  if (Max_domain_element_in_input >= order)
-    fatal_error("mace4: domain_element too big");
+  if (Max_domain_element_in_input >= order) {
+    if (flag(Opt->arithmetic)) {
+      if (!ok_for_arithmetic(clauses, order))
+	return SEARCH_DOMAIN_OUT_OF_RANGE;
+    }
+    else
+      return SEARCH_DOMAIN_OUT_OF_RANGE;
+  }
 
   Domain_size = order;
 
@@ -863,7 +894,11 @@ Mace_results mace4(Plist clauses, Mace_options opt)
     clock_start(Mace4_clock);
     rc = mace4n(clauses, n);
     if (rc == SEARCH_MAX_DOMAIN_SECONDS) {
-      printf("\n====== Domain %d terminated by max_seconds_per. ======\n",n);
+      printf("\n====== Domain size %d terminated by max_seconds_per. ======\n",n);
+      rc = SEARCH_GO_NO_MODELS;
+    }
+    else if (rc == SEARCH_DOMAIN_OUT_OF_RANGE) {
+      printf("\n====== Domain size %d skipped because domain elememt too big. ======\n",n);
       rc = SEARCH_GO_NO_MODELS;
     }
     clock_stop(Mace4_clock);
