@@ -23,6 +23,8 @@
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <unistd.h>
+#include <float.h>
+#include <math.h>
 #include <setjmp.h>  /* Yikes! */
 
 // Private definitions and types
@@ -206,26 +208,24 @@ Prover_options init_prover_options(void)
   p->random_seed       = init_parm("random_seed",          0,     -1,INT_MAX);
   p->eval_limit        = init_parm("eval_limit",        1024,     -1,INT_MAX);
 
-  p->max_weight =       init_parm("max_weight",          100,INT_MIN,INT_MAX);
   p->max_depth =        init_parm("max_depth",            -1,     -1,INT_MAX);
   p->lex_dep_demod_lim =init_parm("lex_dep_demod_lim",    11,     -1,INT_MAX);
   p->max_literals =     init_parm("max_literals",         -1,     -1,INT_MAX);
   p->max_vars =         init_parm("max_vars",             -1,     -1,INT_MAX);
   p->demod_step_limit = init_parm("demod_step_limit",   1000,     -1,INT_MAX);
-  p->demod_size_limit = init_parm("demod_size_limit",   1000,     -1,INT_MAX);
+  p->demod_increase_limit = init_parm("demod_increase_limit",1000,-1,INT_MAX);
   p->backsub_check    = init_parm("backsub_check",       500,     -1,INT_MAX);
 
-  p->variable_weight =  init_parm("variable_weight",       1,INT_MIN,INT_MAX);
-  p->constant_weight =  init_parm("constant_weight",       1,INT_MIN,INT_MAX);
-  p->not_weight =       init_parm("not_weight",            0,INT_MIN,INT_MAX);
-  p->or_weight =        init_parm("or_weight",             0,INT_MIN,INT_MAX);
-  p->sk_constant_weight=init_parm("sk_constant_weight",    1,INT_MIN,INT_MAX);
-  p->prop_atom_weight = init_parm("prop_atom_weight",      1,INT_MIN,INT_MAX);
-  p->skolem_penalty =   init_parm("skolem_penalty",        1,      0,INT_MAX);
-  p->nest_penalty =     init_parm("nest_penalty",          0,      0,INT_MAX);
-  p->depth_penalty =    init_parm("depth_penalty",         0,INT_MIN,INT_MAX);
-  p->var_penalty =      init_parm("var_penalty",           0,INT_MIN,INT_MAX);
-  p->default_weight =   init_parm("default_weight",  INT_MAX,INT_MIN,INT_MAX);
+  p->variable_weight =  init_floatparm("variable_weight",       1.0,-DBL_LARGE,DBL_LARGE);
+  p->constant_weight =  init_floatparm("constant_weight",       1.0,-DBL_LARGE,DBL_LARGE);
+  p->not_weight =       init_floatparm("not_weight",            0.0,-DBL_LARGE,DBL_LARGE);
+  p->or_weight =        init_floatparm("or_weight",             0.0,-DBL_LARGE,DBL_LARGE);
+  p->sk_constant_weight=init_floatparm("sk_constant_weight",    1.0,-DBL_LARGE,DBL_LARGE);
+  p->prop_atom_weight = init_floatparm("prop_atom_weight",      1.0,-DBL_LARGE,DBL_LARGE);
+  p->nest_penalty =     init_floatparm("nest_penalty",          0.0,     0.0,DBL_LARGE);
+  p->depth_penalty =    init_floatparm("depth_penalty",         0.0,-DBL_LARGE,DBL_LARGE);
+  p->var_penalty =      init_floatparm("var_penalty",           0.0,-DBL_LARGE,DBL_LARGE);
+  p->default_weight =   init_floatparm("default_weight",  DBL_LARGE,-DBL_LARGE,DBL_LARGE);
 
   p->sos_limit =        init_parm("sos_limit",         20000,     -1,INT_MAX);
   p->sos_keep_factor =  init_parm("sos_keep_factor",       3,      2,10);
@@ -236,6 +236,12 @@ Prover_options init_prover_options(void)
   p->report =           init_parm("report",               -1,     -1,INT_MAX);
   p->report_stderr =    init_parm("report_stderr",        -1,     -1,INT_MAX);
 
+  // FLOATPARMS:
+  //  internal name      external name           default    min      max )
+
+  p->max_weight =   init_floatparm("max_weight",  100.0, -DBL_LARGE, DBL_LARGE);
+
+
   // STRINGPARMS:
   // (internal-name, external-name, number-of-strings, str1, str2, ... )
   // str1 is always the default
@@ -353,11 +359,11 @@ Prover_options init_prover_options(void)
 
   // auto_limits
 
-  flag_parm_dependency(p->auto_limits,    TRUE,  p->max_weight,        100);
-  flag_parm_dependency(p->auto_limits,    TRUE,  p->sos_limit,       20000);
+  flag_floatparm_dependency(p->auto_limits,  TRUE, p->max_weight,    100.0);
+  flag_parm_dependency(p->auto_limits,       TRUE, p->sos_limit,     20000);
 
-  flag_parm_dependency(p->auto_limits,    FALSE,  p->max_weight,    INT_MAX);
-  flag_parm_dependency(p->auto_limits,    FALSE,  p->sos_limit,          -1);
+  flag_floatparm_dependency(p->auto_limits, FALSE, p->max_weight, DBL_LARGE);
+  flag_parm_dependency(p->auto_limits,      FALSE, p->sos_limit,         -1);
 
   // automatic
 
@@ -378,9 +384,8 @@ Prover_options init_prover_options(void)
   flag_flag_dependency(p->auto2, TRUE, p->automatic,                 TRUE);
   flag_parm_dependency(p->auto2, TRUE, p->new_constants,            1);
   flag_parm_dependency(p->auto2, TRUE, p->fold_denial_max,          3);
-  flag_parm_dependency(p->auto2, TRUE, p->max_weight,             200);
+  flag_floatparm_dependency(p->auto2, TRUE, p->max_weight,      200.0);
   flag_parm_dependency(p->auto2, TRUE, p->nest_penalty,             1);
-  flag_parm_dependency(p->auto2, TRUE, p->skolem_penalty,           3);
   flag_parm_dependency(p->auto2, TRUE, p->sk_constant_weight,       0);
   flag_parm_dependency(p->auto2, TRUE, p->prop_atom_weight,         5);
   flag_flag_dependency(p->auto2, TRUE, p->sort_initial_sos,       TRUE);
@@ -396,9 +401,8 @@ Prover_options init_prover_options(void)
   flag_flag_dep_default(p->auto2, FALSE, p->automatic);
   flag_parm_dep_default(p->auto2, FALSE, p->new_constants);
   flag_parm_dep_default(p->auto2, FALSE, p->fold_denial_max);
-  flag_parm_dep_default(p->auto2, FALSE, p->max_weight);
+  flag_floatparm_dep_default(p->auto2, FALSE, p->max_weight);
   flag_parm_dep_default(p->auto2, FALSE, p->nest_penalty);
-  flag_parm_dep_default(p->auto2, FALSE, p->skolem_penalty);
   flag_parm_dep_default(p->auto2, FALSE, p->sk_constant_weight);
   flag_parm_dep_default(p->auto2, FALSE, p->prop_atom_weight);
   flag_flag_dep_default(p->auto2, FALSE, p->sort_initial_sos);
@@ -1084,9 +1088,9 @@ void clause_wt_with_adjustments(Topform c)
     clock_stop(Clocks.hints);
   }
 
-  if (c->weight > parm(Opt->default_weight) && 
-      c->weight <= parm(Opt->max_weight))
-    c->weight = parm(Opt->default_weight);
+  if (c->weight > floatparm(Opt->default_weight) && 
+      c->weight <= floatparm(Opt->max_weight))
+    c->weight = floatparm(Opt->default_weight);
 }  /* clause_wt_with_adjustments */
 
 /*************
@@ -1130,7 +1134,7 @@ void cl_process_simplify(Topform c)
     clock_start(Clocks.demod);
     demodulate_clause(c,
 		      parm(Opt->demod_step_limit),
-		      parm(Opt->demod_size_limit),
+		      parm(Opt->demod_increase_limit),
 		      !flag(Opt->quiet),
 		      flag(Opt->lex_order_vars));
     if (flag(Opt->print_gen)) {
@@ -1164,7 +1168,6 @@ static
 void cl_process_keep(Topform c)
 {
   Stats.kept++;
-  statistic_actions("kept", clause_ids_assigned());  /* Note different stat */
   if (!c->normal_vars)
     renumber_variables(c, MAX_VARS);
   if (c->id == 0)
@@ -1172,6 +1175,8 @@ void cl_process_keep(Topform c)
   mark_parents_as_used(c);
   mark_maximal_literals(c->literals);
   mark_selected_literals(c->literals, stringparm1(Opt->literal_selection));
+  if (c->matching_hint != NULL)
+    keep_hint_matcher(c);
 
   if (flag(Opt->print_clause_properties))
       c->attributes = set_term_attribute(c->attributes,
@@ -1184,9 +1189,10 @@ void cl_process_keep(Topform c)
   }
   else if (flag(Opt->print_labeled) &&
 	   get_string_attribute(c->attributes, Att.label, 1)) {
-    printf("\nkept (wt=%d): ", c->weight);
+    printf("\nkept:      ");
     fwrite_clause(stdout, c, CL_FORM_STD);
   }
+  statistic_actions("kept", clause_ids_assigned());  /* Note different stat */
 }  // cl_process_keep
 
 static
@@ -1237,7 +1243,7 @@ BOOL skip_black_white_tests(Topform c)
   return (!Glob.searching ||
 	  c->used ||
 	  restricted_denial(c) ||
-	  (c->matches_hint && !flag(Opt->limit_hint_matchers)));
+	  (c->matching_hint  != NULL && !flag(Opt->limit_hint_matchers)));
 }  /* skip_black_white_tests */
 
 static
@@ -1252,7 +1258,7 @@ BOOL cl_process_delete(Topform c)
     return TRUE;  // delete
   }
 
-  clause_wt_with_adjustments(c);  // possibly sets c->matches_hint
+  clause_wt_with_adjustments(c);  // possibly sets c->matching_hint
 
   // White-black tests
 
@@ -1631,12 +1637,6 @@ void limbo_process(BOOL pre_search)
       // do not index_clashable!  disable_clause should not unindex_clashable!
       clist_append(c, Glob.usable);
       index_back_demod(c, INSERT, Clocks.index, flag(Opt->back_demod));
-#if 0
-      if (flag(Opt->print_initial_clauses)) {
-	printf("\nrestricted denial: (wt=%d): ", c->weight);
-	fwrite_clause(stdout, c, CL_FORM_STD);
-      }
-#endif
     }
     else {
       // Move to Sos and index to be found for back demodulation.
@@ -1842,8 +1842,12 @@ void make_inferences(void)
     }
     
     if (flag(Opt->print_given)) {
-      printf("\ngiven #%d (%s,wt=%d): ",
-	     Stats.given, selection_type, given_clause->weight);
+      if (given_clause->weight == round(given_clause->weight))
+	printf("\ngiven #%d (%s,wt=%d): ",
+	       Stats.given, selection_type, (int) given_clause->weight);
+      else
+	printf("\ngiven #%d (%s,wt=%.3f): ",
+	       Stats.given, selection_type, given_clause->weight);
       fwrite_clause(stdout, given_clause, CL_FORM_STD);
     }
 
@@ -2099,16 +2103,15 @@ void init_search(void)
   init_actions(Glob.actions,
 	       rebuild_sos_index, done_with_search, infer_outside_loop);
   init_weight(Glob.weights,
-	      parm(Opt->variable_weight),
-	      parm(Opt->constant_weight),
-	      parm(Opt->not_weight),
-	      parm(Opt->or_weight),
-	      parm(Opt->sk_constant_weight),
-	      parm(Opt->prop_atom_weight),
-	      parm(Opt->skolem_penalty),
-	      parm(Opt->nest_penalty),
-	      parm(Opt->depth_penalty),
-	      parm(Opt->var_penalty));
+	      floatparm(Opt->variable_weight),
+	      floatparm(Opt->constant_weight),
+	      floatparm(Opt->not_weight),
+	      floatparm(Opt->or_weight),
+	      floatparm(Opt->sk_constant_weight),
+	      floatparm(Opt->prop_atom_weight),
+	      floatparm(Opt->nest_penalty),
+	      floatparm(Opt->depth_penalty),
+	      floatparm(Opt->var_penalty));
 
   if (Glob.given_selection == NULL)
     Glob.given_selection = selector_rules_from_options(Opt);
